/*
 * libnqs/comms/analyzercm.c
 * 
 * DESCRIPTION:
 *
 *	Completely analyze and diagnose a request completion
 *	code.
 *
 *
 *	Original Author:
 *	-------
 *	Brent A. Kingsbury, Sterling Software Incorporated.
 *	April 28, 1986.
 */

#include <libnqs/license.h>
#include <libnqs/proto.h>
#include <libnqs/informcc.h>		/* NQS information completion codes */
#include <libnqs/requestcc.h>		/* NQS request completion codes */

static void applypreamble ( FILE *stream, char *prefix );
static void boundpreamble ( FILE *stream, char *prefix );
static void routefaipreamble ( FILE *stream, char *prefix );

/*** analyzercm
 *
 *
 *	void analyzercm():
 *
 *	Completely analyze and diagnose a request completion
 *	code.
 */
void analyzercm (
	long code,		/* Completion code */
	FILE *stream,		/* Output stream */
	char *prefix)		/* Output prefix */
{

	register short reason;		/* Reason bits of completion code */

  	/* debugging code added in 3.41.0 */
  	assert (stream != NULL);
  	assert (prefix != NULL);
  
	reason = (code & XCI_REASON_MASK);
	if (reason < 0 || reason > (RCM_MAXRCM & XCI_REASON_MASK) ||
	   (code & XCI_TRANSA_MASK)) {
		/*
		 *  The request completion code is invalid.
		 */
		reason = (RCM_UNDEFINED & XCI_REASON_MASK);
		code = RCM_UNDEFINED;
	}
	else if (reason == (RCM_UNDEFINED & XCI_REASON_MASK)) {
		/*
		 *  Clear information bits if undefined request completion
		 *  code.
		 */
		code &= ~XCI_INFORM_MASK;
	}
	/*
	 *  Analyze and describe a specific request completion code.
	 */
	rcmmsgs (code, stream, prefix);
	/*
	 *  Analyze and describe any additional information bits
	 *  set in the request completion code.
	 */
	if (reason == RCM_DELIVERFAI || reason == RCM_DELIVERRETX) {
		fputc ('\n', stream);
		fputs (prefix, stream);
		fputs ("Transaction failure reason:\n", stream);
		fflush (stream);	/* For output to logfile */
		tcmmsgs (code >> 10, stream, prefix);
	}
	else if (reason == RCM_STAGEOUTBAK || reason == RCM_STAGEOUTFAI) {
		fputc ('\n', stream);
		fputs (prefix, stream);
		fputs ("Transaction failure reason at primary destination:\n",
			stream);
		fflush (stream);	/* For output to logfile */
		tcmmsgs (code >> 10, stream, prefix);
	}
	else if (reason == RCM_ROUTED && reason == RCM_ROUTEDLOC &&
		 reason == RCM_DELIVERED) {
		/*
		 *  For the moment, we allow RCM_DELIVERED
		 *  to have quota information bits in this
		 *  ****TRIAGE**** implementation.
		 */
		tcimsgs (code, stream, prefix, boundpreamble);
	}
	else if (reason == RCM_ROUTEFAI || reason == RCM_ROUTERETX) {
		rcimsgs (code, stream, prefix, routefaipreamble);
	}
	else if (reason == RCM_UNABLETOEXE) {
		tcimsgs (code, stream, prefix, applypreamble);
	}
}


/*** applypreamble
 *
 *
 *	void applypreamble():
 *	Display limit application preamble.
 */
static void applypreamble (FILE *stream, char *prefix)
{
	fputc ('\n', stream);
	fputs (prefix, stream);
	fputs ("Problem enforcing the following limit(s):\n\n", stream);
	fflush (stream);
}


/*** boundpreamble
 *
 *
 *	void boundpreamble():
 *	Display bounded limit preamble.
 */
static void boundpreamble (FILE *stream, char *prefix)
{
	fputc ('\n', stream);
	fputs (prefix, stream);
	fputs ("In order to successfully queue or deliver the request,\n",
		stream);
	fputs (prefix, stream);
	fputs ("it was necessary to alter the following explicit request\n",
		stream);
	fputs (prefix, stream);
	fputs ("quota limit(s), binding them to values within the ranges\n",
		stream);
	fputs (prefix, stream);
	fputs ("enforceable at the execution machine:\n\n", stream);
	fflush (stream);		/* For output to logfile */
}


/*** routefaipreamble
 *
 *
 *	void routefaipreamble():
 *	Display pipe queue request routing failure preamble.
 */
static void routefaipreamble (FILE *stream, char *prefix)
{
	fputc ('\n', stream);
	fputs (prefix, stream);
	fputs ("The request could not be routed to any of the possible\n",
		stream);
	fputs (prefix, stream);
	fputs (
	  "pipe queue destinations because of the following reason(s):\n\n",
	   stream);
	fflush (stream);		/* For output to logfile */
}
